/*
*   $Id: CCMerchantCustom.c,v 1.18 1998/04/03 15:03:55 vchen Exp $
*
*   System     :   Merchant Connection Kit
*
*   Author     :   CYCH Development staff
*   Description:   Module of customizable functions shared by MCK CGI scripts
*   Notes      :   Each merchant should have a copy of this which is customized
*                  to integrate with the merchant storefront
*
*                         COPYRIGHT NOTICE
*
*   The contents of this file is protected under the United States
*   copyright laws as an unpublished work, and is confidential and
*   proprietary to CyberCash, Inc.  Its use or disclosure in whole or in
*   part without the expressed written permission of CyberCash, Inc. is
*   prohibited.
*
*   (c) Copyright 1993-97 by CyberCash, Inc.  All rights reserved.
*
*/                                                                       

/* Warning!  pointer arithmetic here that may not work with */
/*           multi-byte character sets!                     */


#include <stdlib.h>

#ifdef NO_STDLIB_H
char *getenv();
#endif

#ifndef WIN32
#include <unistd.h>
#else
 typedef long pid_t;
 #include <process.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>

#include <ctype.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <time.h>

#include "CCErrno.h"
#include "CCNVlist.h"
#include "CCMckLib.h"
#include "CCString.h"
#include "CCBlock.h"

#include "CCConfig.h"
#include "CCMerchantTest.h"
#include "CCMerchantCustom.h"

   /* MAXTMPLINE should be bigger than biggest line you expect in a template. */
#define MAXTMPLINE 1024

THREAD char msgBuf[MAXTMPLINE];

/*============================================================================
 * Define oprstional constants
 *===========================================================================*/
#define PRICESIZE		 40		 /* length of price				*/
#define CRCY_UNIT_LEN 3			/* max length of currency such as usd, yen */

/*============================================================================
 * Merchant constants
 *===========================================================================*/

/*##########################################################
 * Constants to customize
 *##########################################################*/

/* --- Amount format, assuming 2 decimal places ---*/
const char _AmountFormat[] = "%s %3.2f";	/* e.g. usd 12.34 */


/* template is understood to be a full path name... */
void ReportFailure(const char* message, const char* orderId, 
		const char* templ)
{
	CCNVlistPtr tokens = NULL;

	CCLogError(message);
	tokens = CCNewNVlist();
		
	if (tokens == NULL) {
		/* indicates No Memory!: CC_ENO_MEMORY */
		CCLogError("%s: ReportFailure(): %s", 
			progname, CCGetErrorMessage(CC_ENO_MEMORY));
		return;  /* can't do anything more... */
	}

	if (isDefinedString(orderId)) {
		CCAddPairNVlist( tokens, "ORDERID", orderId);
	}
	CCAddPairNVlist( tokens, "MESSAGE", message);
	PrintTemplate(templ, tokens);
		
	CCDeleteNVlist(tokens);
}

/**
 * EXPERSHOP
 * Removed GenerateOrderId function
 * (Order IDs are expershop-generated)
 */
/*
char *GenerateOrderId(char *orderid) 
*/


/*
**********************************
* LoadPaymentInfo
*     paymentNVList 
*		= LoadPaymentInfo(orderId, query, paymentNVList, problemTemplate);
*
* Inputs:
*     orderId  -- order identifier... key to recovering payment info
*     query  -- pointer to the CGI input parameters
*     paymentNVList  -- pointer to the array of payment
*                          request arguments that we are building
*     problemTemplate -- template to use for error message if this fails
*
*  Returns paymentNVList
*          Notifies, logs then returns NULL if there is a problem
*  
*  This collects order related payment info and loads it into
*   the vector of payment arguments (paymentNVList)...
*   This sets a number of required and optional "mo." fields in the
*   payment request message.
*
* Our sample implementation gets the payment data from a flat file log
* It uses an orderId for the key.  This orderId is passed to the CGI as a query
*  parameter 
*
* Your implementation will probably store the payment data some other way.
***************************
*/

CCNVlistPtr LoadPaymentInfo (
	char* orderId, CCNVlistPtr query, CCNVlistPtr paymentNVList, 
	char* problemTemplate)
{
	char *str;
	int errcode;
	int found; 

		/* for price validation and canonicalization */
	char *price;
    	char currency[4];
	double amount;
 		/* this has got to be at least a big as the largest                 */
		/* string you ever expect to see for an amount plus 5 spaces        */
		/* for the three character currency code, a balnk and the null byte */
	char canonicalPrice[MAXTMPLINE]; 


	OrderRecPtr orderInfo = NULL;


	/* Test this to make sure it's a valid one... */
	if (!isDefinedString(orderId)) {
		/* CC_ENO_ORDERID */
		sprintf(msgBuf, "%s: %s\n", progname, 
			CCGetErrorMessage(CC_ENO_ORDERID)); 
		ReportFailure(msgBuf, "", problemTemplate);
		CCDeleteNVlist(paymentNVList);
		paymentNVList = NULL;

		goto BAIL;
	}


	CCAddPairNVlist(paymentNVList, "mo.order-id", orderId);

	/* Now read anything you need from the form into  */
	/* the appropriate message arguments: */

	/*
	 * EXPERSHOP
	 * The Order Info should be in the HTTP request (in the query arg).
	 */

	/* At this point we **SHOULD** have a price, etc... */
	/* Validate the price for syntax and currency choice */
	/* If validation fails, it will return a dup'd error message */
	/* else returns null */

	/* canonical representation is returned here too */
	
	price = CCGetValNVlist(query, "mo.price");

	errcode = ValidatePrice(price, currency, &amount, canonicalPrice); 
	if (errcode) {
		 /* bad Price: CC_EBAD_AMOUNT */
		if (errcode == CC_EBAD_AMOUNT) {
			sprintf(msgBuf,"%s: Order %s,  %s .\n", progname, 
				orderId, CCGetErrorMessage(CC_EBAD_AMOUNT)); 
		} 
		else {
			sprintf(msgBuf,"%s: Order %s,  %s: [%d - %s] .\n", progname, 
				orderId, CCGetErrorMessage(CC_EBAD_AMOUNT), 
				errcode, CCGetErrorMessage(errcode));
		}
		
		ReportFailure(msgBuf, orderId, problemTemplate);

		CCDeleteNVlist(paymentNVList);
		paymentNVList = NULL;

		goto BAIL;
	}

	/* replace the input price with a canonical representation */
	free (price); /* it's been strdup'd */
	price = strdup(canonicalPrice);


	/*
	* If you need to manipulate the price, try this.
	*  After the call to ValidatePrice, 
	* currency is a currency string (i.e. "usd")
	* amount is a NUMERICAL amount value that you can
	* do arithmetic with....
	* 
	* if you want to apply a discount, or adjust the price in
	* some other way, this is the place to do it
	* 
	* amount is the variable that you want to adjust
	* For example:
	*    amount = amount - 1.00;   * take 1.00 off
	*    amount = amount * 0.90;   * 10% discount
	*
	* Now, reformat the price correctly
	*
	*  sprintf(amtbuf,"%.2f", amount);
	*  sprintf(pricebuf, "%3s %.2f",currency,amount);
	*/
	 
	/* Here is the price we are charging... */
	CCAddPairNVlist(paymentNVList, "mo.price", price);
	free(price);
	
        if((str = CCGetValNVlist(query, "mo.payload")) != NULL) {
		/*
		* This field is optional... 
		* If you are selling a file as a payload, set this to
		*  the name of the file you wish to sell
		*/
		CCAddPairNVlist(paymentNVList, "mo.payload", str);
		free(str);
	}
	
	if((str = CCGetValNVlist(query, "mo.product-descr")) != NULL) {
	
		/* 
		* productDescr is a description of what is being sold: from the Order Data
		* This is optional
		*/
	
		CCAddPairNVlist(paymentNVList, "mo.product-descr", str);
		free(str);
	}
	
	if((str = CCGetValNVlist(query, "mo.order-valid-till")) != NULL) {
	
		/*
		* this is a date after which your fulfillment center will 
		* reject the  notification/ fulfillment because the order is too stale.
		*
		* The format of this string is entirely up to you.  If you use
		*  This, you will need to add code to enforce the date in your
		* notification/ fulfillment scripts.
		
		* This is optional
		*/
	
		CCAddPairNVlist(paymentNVList, "mo.order-valid-till", str);
		free(str);
	}
	
	
	if((str = CCGetValNVlist(query, "mo.customer-id")) != NULL) {
	
		/* Some identifier you assigned to identify this customer. */
		/*  The format is up to you.  This will be passed in the POP. */
		/* This is optional */
	
		CCAddPairNVlist(paymentNVList, "mo.customer-id", str);
		free(str);
	}

BAIL:
	return paymentNVList;
}


/**
 * EXPERSHOP
 * TBD ReadPaymentInfo for mswalletpay.c
 */
OrderRecPtr ReadPaymentInfo ( char* orderId, char* problemTemplate)
{
	return NULL;
}



/*
******************************
*  Routines that validate price syntax and manipulate prices
*
*             ******
*
*  Most merchants will never need to edit this code.
*  It is in this section because some merchants may handle multiple
* currencies -- and *THEY* will have to test for the acceptable ones here.
*
* Also:  this expects money to be in the format:
*   aaa 9+.9+
* which is to say, a 3 letter ISO currency code, a blank and a decimal part
* of at least two digits of precision.
* This does not cover currencies like the Japanese Yen, which do not use
* a decimal part to express fractional Yen.
*
* If a merchant accepts yen, the price validation may need to be adjusted
*
******************************
*/

/*
*****************************
* ValidatePrice
*
* Test the price string for syntactic validity.
* Check for acceptable currency
*
* INPUTS:
* The incoming price is not altered.
*
* OUTPUTS: 
* currencyOut - a string giving the ISO currency code for this price.
*         The string passed in must be at least 4 chars big.
* amountOut - a double that is the amount in a from you can calculate with.
* canonicalPrice - The price represented as the Cash Register 
*        would like to see it.  Format is: 'usd 9.99'
*        namely: a lower-case 3 letter ISO currency code,
*                a blank
*                at least one digit, a decimal point and two digits after that
*        The buffer you pass in has got to be at least a big as the largest 
*		 string you ever expect to see for an amount plus 5 spaces
*		 for the three character currency code, a blank and the null byte
*
*  NOTE: The canonical representation has two decimal places.
*    If you need more or less, you will need to change this.
*
* this returns an error code. 0 if OK
*
*****************************
*/

int ValidatePrice (const char* priceIn, 
	char* currencyOut, double *amountOut, char* canonicalPrice)
{
	char* cp, *cp2;
	char  currency[MAXTMPLINE]; /* extravagant, but allows for BAD errors */ 
	char  amount[MAXTMPLINE];
	char  price[MAXTMPLINE];

		/* legal chars in a price .. lower case before test*/
	char* legals = " \t.0123456789abcdefghijklmnopqrstuvwxyz";
	int errcode = CC_SUCCESS;
	int len;

	if (Config == NULL) {  /* default currency is here */
		return CC_ENULL_CONFIG;
	}
		/* sanity check here... */
    if ((NULL == currencyOut) 
		|| (NULL == amountOut) 
		|| (NULL == canonicalPrice)
	   ) 
	{
        return CC_EBAD_PARAMS;
	}


	if (strlen(priceIn) >= MAXTMPLINE) {
		/* this would be a price larger than the US national debt */
		/* if it was *remotely* properly formatted.  reject it now! */

		errcode = CC_EBIG_AMOUNT;
		goto CLEANUP;
	}

		/* work on the copy... don't mess with the original */
	strcpy(price, priceIn);

	if (price) {
		Trim(price);
	}
	else {
		errcode = CC_ENO_AMOUNT;
		goto CLEANUP;
	}

	/* if this is empty, complain */
	if (strlen(price) == 0) {
		errcode = CC_ENO_AMOUNT;
		goto CLEANUP;
	}

	lowerString(price);

	/* are there undesirable characters in here? */
	if (strspn(price, legals) != strlen(price)) {
		/* illegal characters */
		errcode = CC_EBAD_AMOUNT; 
		goto CLEANUP;
	}
		
	if (isalpha(price[0])) {   /* we can expect a currency code here */
		/* --- Get currency --- */

		cp = price;
		len = 0;
		while (isalpha(*cp)) {
			len++;
			cp++;
		}
	
		/* cp points to first non-alpha */
		/* Copy this... */
		strncpy(currency, price, len);
		
		/* close it */
		currency[len] = '\0';

		/* Now... find first non-blank */
		while (isspace(*cp)) {
			cp++;
		}

		/* cp points to first non-blank */
		strcpy(amount, cp);
	}
	else if (isdigit(price[0])) {
		/* no currency.. so default it */
		strcpy(currency, Config->defaultCurrency);
		strcpy(amount, price);
	}
	else {
		/* Format violation ... if no currency, expect a digit here*/
		errcode =  CC_EBAD_AMOUNT;
		goto CLEANUP;
	}

	/* sanity check on currency */
	if(CRCY_UNIT_LEN != strlen(currency)) {
		errcode =  CC_EBAD_CURRENCY;
		goto CLEANUP;
	}

	/* Now... look at the price component */
	cp = strchr(amount, '.');  /* decimal point */
	if (NULL == cp) {
		errcode = CC_EBAD_AMOUNT;
		goto CLEANUP;
	}

	/* Sigh... need all digits and ONLY one dec. point */
	for (cp2 = amount; cp2 != cp; cp2++) {
		if (!isdigit(*cp2)) {
			errcode = CC_EBAD_AMOUNT;
			goto CLEANUP;
		}
	}

	len =0;
	for (cp2 = cp+1; *cp2 != '\0'; cp2++) {
		len++;
		if (!isdigit(*cp2)) {
			errcode = CC_EBAD_AMOUNT;
			goto CLEANUP;
		}
	}
	if (len <2) {
		errcode = CC_EBAD_AMOUNT;
		goto CLEANUP;
	}

	errno = 0;
	*amountOut = atof(amount);

	if (errno) {
		errcode = CC_EBAD_AMOUNT;
		goto CLEANUP;
	}
	else if (*amountOut <= 0.00) {
		errcode = CC_ENEG_AMOUNT;
		goto CLEANUP;
	}

	/*
	* This looks like a valid currency code... is the  currency acceptable?
	* At this point, check to see if this currency is acceptable to you...
	* If you accept the default and some others, add them to the test like this:
	* if ( 
	*	  (strcmp (currency, Config->defaultCurrency) != 0)
	*     && (strcmp(currency,"frf") != 0)
    *     && (strcmp(currency,"cad") != 0)
	*  ) {
	*/
	
	if (strcmp(currency, Config->defaultCurrency) != 0) 
	{
		 /* YOICKS! this isn't a currency we can use! */
		 /* bad currency */
		errcode = CC_EREJECT_CURRENCY;
	}

	strcpy(currencyOut, currency);

CLEANUP:
	if (errcode != 0){
		/* zap it */
		strcpy(canonicalPrice, "");
	}
	else {
		/* Build a canonical price string */
		/* Note: we are ASSUMING this buffer is big enough! */

		sprintf(canonicalPrice, "%3s %.2f", currencyOut, *amountOut);
	}

	return errcode;
}

int SendPayload (char* payload, char* orderId, char *problemTemplate)
{
	
	char* suffix = NULL;
	
	char buf[MAXTMPLINE+1]; /* maybe ok? add one for a null byte at end*/

	struct stat filestat;
	off_t payloadSize;
	size_t total_bytes_sent, num_read, outix, num_out;

	char* errTemplate = NULL;
	size_t etBufSz = 0;

	char* notForSaleTemplate = NULL;
	size_t nfsBufSz = 0;

	char* fullPayloadPath = NULL;
	size_t pathBufSz = 0;

	char* mimeType = NULL;

	int errcode = CC_SUCCESS;

	FILE* payload_fd = NULL;

	buf[0] = 0; 

        /* needs to initialize for NT */
        memset(buf, 0x00, MAXTMPLINE+1);

	/* Do we have configuration data? */
	if (NULL == Config)
	{   
		sprintf(msgBuf, "%s: %s", progname, CCGetErrorMessage(CC_ENULL_CONFIG));

		ReportFailure(msgBuf, orderId, problemTemplate);

			/* nothing allocated, nothing to CLEANUP */
		return  CC_ENULL_CONFIG;
	}

	/* clean up filename so there are no unsafe characters */
	/* in case it gets to the command line   */

	payload = CleanString(payload);

	suffix = strrchr(payload, '.');  /* look for file type */

	if (NULL == suffix
		|| suffix[1] == '\0') 
	{   /* empty suffix */
		
		/* No file extension... we can't do anything here */

		sprintf(msgBuf, "%s: %s (%s)", progname, 
			CCGetErrorMessage(CC_EBAD_PAYLOAD), payload);

		ReportFailure(msgBuf, orderId, problemTemplate);

			/* nothing allocated, nothing to CLEANUP */
		return  CC_EBAD_PAYLOAD;
	}

	suffix++;  /* don't want the dot.. */
	mimeType = GetMimeType(suffix, Config->mimeTypeFile); /* dup'd */


		/* Filenames are relative to the Payload Dir */

	fullPayloadPath = catString(fullPayloadPath, &pathBufSz,
			Config->payloadDir);
   	fullPayloadPath = catString(fullPayloadPath, &pathBufSz, 
			payload);

		/* want to get the legth of this file... */
	if (stat(fullPayloadPath, &filestat) != 0) {
		/* Groan! can't stat this.. either not there or forbidden! */
		CCLogError ("%s: Payload  - %s, errno = %d.",progname, 
			CCGetErrorMessage(CC_ESTAT_FAILED), errno);
		
		/* die here.. */
		sprintf(msgBuf, "%s: %s (%s)", progname, 
			CCGetErrorMessage(CC_ENO_PAYLOAD), payload);

		ReportFailure(msgBuf, orderId, problemTemplate);
		errcode = CC_ENO_PAYLOAD;
		goto CLEANUP;
	}
	else {
		payloadSize = filestat.st_size;   /* file size in bytes */
	}
	
	payload_fd = fopen(fullPayloadPath,"rb");
	if (!payload_fd) {
		/* OOOPS! can't open the payload */

		sprintf(msgBuf, "%s: %s (%s)", progname, 
			CCGetErrorMessage(CC_ENO_PAYLOAD), payload);

		ReportFailure(msgBuf, orderId, problemTemplate);

		errcode = CC_ENO_PAYLOAD;
		goto CLEANUP;
	}

	printf("Content-Type: %s\r\n", mimeType);
	printf("Content-Length: %d\r\n", payloadSize);
	printf("\r\n");
	fflush(stdout);

	/* now dump the goods */
    total_bytes_sent = 0;
    while (1) {
        if((num_read=fread(buf,sizeof(char), MAXTMPLINE, payload_fd)) < 1) {
            break;
        }
        outix = 0;
        while(num_read) {
#if (defined(NT) || defined(_WIN32))
            num_out = BinWrite(stdout, &buf[outix],num_read);
#else
            num_out = BinWrite(STDOUT_FILENO, &buf[outix],num_read);
#endif
			if (num_out <= 0) {
				/* Client hung up...(or blew up) */
				goto CLEANUP;
			}
				
            num_read -= num_out;
            outix += num_out;
			total_bytes_sent += num_out;
        }
    }

CLEANUP:
	if (payload_fd) fclose(payload_fd);

	if (errTemplate)        free (errTemplate);
	if (notForSaleTemplate) free (notForSaleTemplate);
	if (fullPayloadPath)    free (fullPayloadPath);
	if (mimeType)           free (mimeType);

	return errcode;
}

