# --*-PERL-*--
#####################################################################
#
# Copyright 1996, CyberCash Inc. All rights reserved.
# Written by Judy Grass
#
# These routines are intended to be used exclusively with the 
# CyberCash secure payment system.
#
# RCSID: $Id: CCMerchantTest.pm,v 1.14 1998/04/01 15:22:07 grass Exp $
#
# This contains merchant configuration parameters
#  And common merchant customizable subroutines used by all of the MCK
#  Payment scripts.
#
# Do not redistribute without written permission from CyberCash Inc.
######################################################################

package CCMerchantTest;

require 5.003;

# The following 4 lines are here only because the current directory for
# Microsoft IIS when running script on browser is always the root
# directory of the web server.
BEGIN {
    push(@INC, '/infosystem/www/htdocs/membres/gibello/CyberCash/test-mck/mck-cgi');
    push(@INC, '/infosystem/www/htdocs/membres/gibello/CyberCash/test-mck/mck-cgi/');
}

require Exporter;
@ISA = (Exporter);
@EXPORT = qw();
@EXPORT_OK = qw(CheckOrderId 
	LogNotification ExtractNotification
	TemplateFailure  $progdir $ConfigFile);
# EXPERSHOP Removed SaveOrderInfo and GetOrderInfo

use CCMckLib3_2 qw(%Config $defaultCurrency InitDebug Clean Trim ValidateOrderId
	urlencode URLdecodeForm
	CCError CCDebug CCDebug2);

use CCMckErrno3_2 qw(MCKGetErrorMessage $E_NoErr
		$E_No_OrderLog
		$E_No_NotifLog
		$E_Order_NotFound
	);

# this is hard coded here... gotta start somewhere!

$OS = 'UNIX';

$progdir = '/infosystem/www/htdocs/membres/gibello/CyberCash/test-mck/mck-cgi/';

if ($OS eq 'NT') {
   $ConfigFile =  $progdir . "conf\\merchant_conf";
}
else {
   $ConfigFile =  $progdir . "conf/merchant_conf";
}
 
################################
# TemplateFailure 
#
# This is here for total melt-down of templating.
# Otherwise, templates will be expanded for messaging.
#
# Please customize the text here if you wish.
#
##########################################

sub TemplateFailure 
{
	my($message) = @_;
	print "Content-type: text/html\r\n\r\n";
	
	print <<"END";
<html>

<head>
<title>Temporary Difficulties</title>
</head>

<body>
<h1>Temporary Difficulties</h1>

<p>CyberCash MCK Test Merchant is experiencing 
temporary difficulties.</p>

$message

<p>Please contact CyberCash MCK Test Merchant
customer service staff at merchant-support\@cybercash.com, 
or try again later.</p>

<p>Sorry for the inconvenience.</p>

</body>
</html>
END

}

#########################################################
# EXPERSHOP: Removed 
#  SaveOrderInfo and GetOrderInfo
####################


##############################
# sub CheckOrderId
#
# When the order was generated, the OrderId was saved.
# Now, check to see if the OrderId on the notification is an OrderId 
# that you recognize.
#
#  See comments in SaveOrderInfo to see how the test script implements
#  the Order Log.
#  
# I'm using a flat file, but this will not scale to a large, complex
#  storefront.  You will need to replace this function to work with
#  the kind of data you store and the method you use to store that data.
#
############################

sub CheckOrderId {
    my($orderId) = @_;
	my($found) = 0;

	# should add locking here...

	unless(open (ORDERDB, "<".$Config{"ORDER_DB"})) {
		return $E_No_OrderLog;  # DB failure
	}
		
	# ** sigh! ** linear search...
	SEARCH:
	while (<ORDERDB>) {
		if (/^$orderId$ORDER_RSEP/) {
			$found = 1;
			last SEARCH;
		}
	}
			
	close(ORDERDB);
  
    return $found; 
}

###################################
#  LogNotification
#
#  Write the POP to the notification log
#  If you want to change the format or what is saved,
#  make the  changes here
#
# We are saving the POP string as it arrived...
#
#  return $status, $code as below.
#   $status is numeric code (see CCMckErrno3_2.pm)
#	$code = "000" is success
#	$code = "3xx" is a file error of some kind
#
########################################

sub LogNotification
{
	my ($POPstr) = @_;

	if(!open(LOG,">>".$Config{"NOTIFICATION_LOG"})) {
		$status = $E_No_NotifLog;
		$code = "300";
		return ($status, $code);
	}
	else {

		print LOG "$POPstr\n";
		close(LOG);

		$status = $E_NoErr;
		$code = "000";
	}
	return ($status, $code);
}

###################################
#  ExtractNotification
#
#  Input:  An order ID	
#  Output: An error code   (0 = OK, > 0  errno: file problem,
#                           -1 if record not found)
#          An error message (if applicable)
#          An associative array containg the fields of the POP record
#  
#  If you change the Notification Log format or what is saved in that log,
#  You will have to make matching changes here...
#
#  Note: The POP string is stored in URL encoded format,
#  as it arrived in the directpay and/or fulfillment cgis...  
#
########################################

sub ExtractNotification
{
	my ($orderId) = @_;
	my (%POP);
	my ($line, $oid, $log);
	my ($message) = "";
	my ($status) = 0;

	%POP=();   # initialize to empty

	$log = $Config{"NOTIFICATION_LOG"};

	if(!open(LOG,"<".$Config{"NOTIFICATION_LOG"})) {
		$status = $!;
		$message = "[$!] ". &MCKGetErrorMessage($E_No_NotifLog) . " ($log)";
		return ($status, $message, %POP);
	}
	
	while (<LOG>) {
		# look for OID in each line
		$line = $_;

		# Do this on the cheap... 
		# find pop.order-id in the string, without URL decoding
		# extract its value  from the raw string.
		# urldecode that
		# if it matches our oid, ***THEN*** we'll unpack the record.

		if ($line =~ /pop.order-id=([^&]+)/ ) 
		{
			$oid = $1;

				# urldecode
			$oid =~ s/\+/ /g;
        	$oid =~ s/%([a-fA-F0-9]{2})/pack("c",hex($1))/ge;

			if ($oid eq $orderId) 
			{
				# this is it!
				%POP = &URLdecodeForm($line);
				close (LOG);
				$message = &MCKGetErrorMessage($E_NoErr);
				return (0, $message, %POP);
			}
		}
		# if no pop.order-id, this is wierd... but ignore it

	}
	$message = &MCKGetErrorMessage($E_Order_NotFound);
	return (-1, $message, %POP);
}


1;
